/*****************************************************************************
* Copyright (c) 2014 Ted John
* OpenRCT2, an open source clone of Roller Coaster Tycoon 2.
*
* This file is part of OpenRCT2.
*
* OpenRCT2 is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.

* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/

#include "track_data.h"

const rct_track_coordinates TrackCoordinates[256] = {
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 0, 64, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 32, 0, 0 },
		{ 0, 0, 0, 32, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 64, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 32, 0, 0, 0 },
		{ 0, 0, 32, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 3, 0, 0, -64, -64 },
		{ 0, 1, 0, 0, -64, 64 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 3, 0, 0, -64, -64 },
		{ 0, 1, 0, 0, -64, 64 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 3, 0, 64, -64, -64 },
		{ 0, 1, 0, 64, -64, 64 },
		{ 0, 3, 64, 0, -64, -64 },
		{ 0, 1, 64, 0, -64, 64 },
		{ 0, 0, 0, 0, -64, -32 },
		{ 0, 0, 0, 0, -64, 32 },
		{ 0, 0, 0, 0, -32, -32 },
		{ 0, 0, 0, 0, -32, 32 },
		{ 0, 3, 0, 0, -32, -32 },
		{ 0, 1, 0, 0, -32, 32 },
		{ 0, 3, 0, 0, -32, -32 },
		{ 0, 1, 0, 0, -32, 32 },
		{ 0, 3, 0, 32, -32, -32 },
		{ 0, 1, 0, 32, -32, 32 },
		{ 0, 3, 32, 0, -32, -32 },
		{ 0, 1, 32, 0, -32, 32 },
		{ 0, 3, 0, 0, 0, 0 },
		{ 0, 1, 0, 0, 0, 0 },
		{ 0, 0, 0, 16, -64, 0 },
		{ 0, 0, 0, 16, -64, 0 },
		{ 0, 0, 0, -16, -64, 0 },
		{ 0, 0, 0, -16, -64, 0 },
		{ 0, 2, 0, 152, -32, 0 },
		{ 0, 2, 0, -152, 32, 0 },
		{ 0, 3, 0, 80, -32, -32 },
		{ 0, 1, 0, 80, -32, 32 },
		{ 0, 3, 0, -80, -32, -32 },
		{ 0, 1, 0, -80, -32, 32 },
		{ 0, 0, 0, 24, 0, 0 },
		{ 0, 0, 0, 24, 0, 0 },
		{ 0, 0, 24, 0, 0, 0 },
		{ 0, 0, 24, 0, 0, 0 },
		{ 0, 0, 0, 96, 32, 0 },
		{ 0, 0, 0, 32, 32, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 0, 64, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 32, 0, 0 },
		{ 0, 0, 0, 32, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 64, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 32, 0, 0, 0 },
		{ 0, 0, 32, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 3, 0, 0, -64, -64 },
		{ 0, 1, 0, 0, -64, 64 },
		{ 0, 0, 0, 0, -64, -32 },
		{ 0, 0, 0, 0, -64, 32 },
		{ 0, 3, 0, 0, -32, -32 },
		{ 0, 1, 0, 0, -32, 32 },
		{ 0, 2, 0, 16, 0, -96 },
		{ 0, 2, 0, 16, 0, 96 },
		{ 0, 2, 16, 0, 0, -96 },
		{ 0, 2, 16, 0, 0, 96 },
		{ 0, 2, 0, 16, 0, -160 },
		{ 0, 2, 0, 16, 0, 160 },
		{ 0, 2, 16, 0, 0, -160 },
		{ 0, 2, 16, 0, 0, 160 },
		{ 0, 3, 0, 64, 0, 0 },
		{ 0, 1, 0, 64, 0, 0 },
		{ 0, 3, 64, 0, 0, 0 },
		{ 0, 1, 64, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 3, 0, 16, -64, -64 },
		{ 0, 1, 0, 16, -64, 64 },
		{ 0, 3, 16, 0, -64, -64 },
		{ 0, 1, 16, 0, -64, 64 },
		{ 0, 3, 0, 16, -64, -64 },
		{ 0, 1, 0, 16, -64, 64 },
		{ 0, 3, 16, 0, -64, -64 },
		{ 0, 1, 16, 0, -64, 64 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 16, 16, -128, 0 },
		{ 0, 0, 0, 88, -96, 0 },
		{ 0, 0, 0, 88, -96, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 88, 0, -96, 0 },
		{ 0, 0, 88, 0, -96, 0 },
		{ 0, 0, 0, -96, -96, 0 },
		{ 0, 0, 0, 240, -160, 0 },
		{ 0, 0, 0, 80, 32, 0 },
		{ 0, 0, 0, 32, 32, 0 },
		{ 0, 0, 32, 0, 32, 0 },
		{ 0, 0, 0, 56, 32, 0 },
		{ 0, 0, 56, 0, 0, 0 },
		{ 0, 0, 0, 56, 0, 0 },
		{ 0, 0, 56, 0, 32, 0 },
		{ 0, 0, 24, 0, 0, 0 },
		{ 0, 7, 0, 0, -64, -32 },
		{ 0, 4, 0, 0, -64, 32 },
		{ 4, 0, 0, 0, -64, 32 },
		{ 4, 1, 0, 0, -32, 64 },
		{ 0, 7, 0, 0, -64, -32 },
		{ 0, 4, 0, 0, -64, 32 },
		{ 4, 0, 0, 0, -64, 32 },
		{ 4, 1, 0, 0, -32, 64 },
		{ 4, 4, 0, 0, -32, 32 },
		{ 4, 4, 0, 16, -32, 32 },
		{ 4, 4, 0, 64, -32, 32 },
		{ 4, 4, 0, 8, -32, 32 },
		{ 4, 4, 0, 32, -32, 32 },
		{ 4, 4, 0, 32, -32, 32 },
		{ 4, 4, 0, 8, -32, 32 },
		{ 4, 4, 16, 0, -32, 32 },
		{ 4, 4, 64, 0, -32, 32 },
		{ 4, 4, 8, 0, -32, 32 },
		{ 4, 4, 32, 0, -32, 32 },
		{ 4, 4, 32, 0, -32, 32 },
		{ 4, 4, 8, 0, -32, 32 },
		{ 4, 4, 0, 24, -32, 32 },
		{ 4, 4, 0, 24, -32, 32 },
		{ 4, 4, 24, 0, -32, 32 },
		{ 4, 4, 24, 0, -32, 32 },
		{ 4, 4, 0, 0, -32, 32 },
		{ 4, 4, 0, 0, -32, 32 },
		{ 4, 4, 0, 0, -32, 32 },
		{ 4, 4, 0, 0, -32, 32 },
		{ 4, 4, 0, 8, -32, 32 },
		{ 4, 4, 0, 8, -32, 32 },
		{ 4, 4, 0, 8, -32, 32 },
		{ 4, 4, 0, 8, -32, 32 },
		{ 4, 4, 8, 0, -32, 32 },
		{ 4, 4, 8, 0, -32, 32 },
		{ 4, 4, 8, 0, -32, 32 },
		{ 4, 4, 8, 0, -32, 32 },
		{ 4, 4, 0, 0, -32, 32 },
		{ 4, 4, 0, 0, -32, 32 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 32, -64, 0 },
		{ 0, 0, 0, 32, -64, 0 },
		{ 0, 0, 0, -32, -64, 0 },
		{ 0, 0, 0, -32, -64, 0 },
		{ 0, 3, 0, 24, -32, -32 },
		{ 0, 1, 0, 24, -32, 32 },
		{ 0, 3, 24, 0, -32, -32 },
		{ 0, 1, 24, 0, -32, 32 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 2, 0, 280, -64, -32 },
		{ 0, 2, 0, 280, -64, 32 },
		{ 0, 2, 0, -280, 64, -32 },
		{ 0, 2, 0, -280, 64, 32 },
		{ 0, 0, 0, -16, -64, 0 },
		{ 0, 0, 0, -16, -64, 0 },
		{ 0, 0, 0, 16, -64, 0 },
		{ 0, 0, 0, 16, -64, 0 },
		{ 0, 2, 0, 120, -32, 0 },
		{ 0, 2, 0, -120, 32, 0 },
		{ 0, 3, 0, 48, -32, -32 },
		{ 0, 1, 0, 48, -32, 32 },
		{ 0, 3, 0, -48, -32, -32 },
		{ 0, 1, 0, -48, -32, 32 },
		{ 0, 2, 0, 32, 0, 0 },
		{ 0, 2, 0, -32, 0, 0 },
		{ 0, 0, 0, 0, -160, 0 },
		{ 0, 0, 0, 0, -160, 0 },
		{ 0, 0, 0, 0, -32, 0 },
		{ 0, 0, 0, 0, -32, 0 },
		{ 0, 0, 0, 0, -32, 0 },
		{ 0, 1, 0, 0, -32, 32 },
		{ 0, 3, 0, 0, -32, -32 },
		{ 0, 2, 0, -96, -96, 0 },
		{ 0, 2, 0, 128, 64, 0 },
		{ 0, 2, 0, -128, -96, 0 },
		{ 0, 3, 0, 16, -32, -32 },
		{ 0, 1, 0, 16, -32, 32 },
		{ 0, 0, 0, 0, -64, 0 },
		{ 0, 0, 0, 0, -64, 0 },
		{ 0, 0, 0, 0, -32, 0 },
		{ 0, 0, 80, 0, 32, 0 },
		{ 0, 0, 240, 0, -160, 0 },
		{ 0, 0, 0, 0, 0, 0 },
		{ 0, 3, 0, 32, -32, -32 },
		{ 0, 1, 0, 32, -32, 32 },
		{ 0, 3, 32, 0, -32, -32 },
		{ 0, 1, 32, 0, -32, 32 },
		{ 0, 3, 0, 64, -64, -64 },
		{ 0, 1, 0, 64, -64, 64 },
		{ 0, 3, 64, 0, -64, -64 },
		{ 0, 1, 64, 0, -64, 64 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 0, 16, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 16, 0, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 0, 8, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 0, 8, 0, 0, 0 },
		{ 0, 3, 0, 96, 0, 32 },
		{ 0, 1, 0, 96, 0, -32 },
		{ 0, 3, 96, 0, 0, 32 },
		{ 0, 1, 96, 0, 0, -32 },
		{ 0, 2, 0, 96, 64, 0 },
		{ 0, 2, 0, -128, -96, 0 },
		{ 0, 2, 0, 128, 64, 0 }
};